@@ -26,8 +26,6 @@ through the code to commit the whitespace formatting.
   FormatDecisionState: main class exported by this module.
 """
 
-import copy
-
 from yapf.yapflib import format_token
 from yapf.yapflib import split_penalty
 from yapf.yapflib import style
@@ -80,11 +78,21 @@ class FormatDecisionState(object):
     self.newline = False
     self.previous = None
     self.column_limit = style.Get('COLUMN_LIMIT')
-    self._MoveStateToNextToken()
 
   def Clone(self):
-    new = copy.copy(self)
-    new.stack = [copy.copy(state) for state in self.stack]
+    """Clones a FormatDecisionState object."""
+    new = FormatDecisionState(self.line, self.first_indent)
+    new.next_token = self.next_token
+    new.column = self.column
+    new.line = self.line
+    new.paren_level = self.paren_level
+    new.start_of_line_level = self.start_of_line_level
+    new.lowest_level_on_line = self.lowest_level_on_line
+    new.ignore_stack_for_comparison = self.ignore_stack_for_comparison
+    new.first_indent = self.first_indent
+    new.newline = self.newline
+    new.previous = self.previous
+    new.stack = [state.Clone() for state in self.stack]
     return new
 
   def __eq__(self, other):
@@ -95,9 +103,9 @@ class FormatDecisionState(object):
             self.column == other.column and
             self.paren_level == other.paren_level and
             self.start_of_line_level == other.start_of_line_level and
-            self.lowest_level_on_line == other.lowest_level_on_line and (
-                self.ignore_stack_for_comparison or
-                other.ignore_stack_for_comparison or self.stack == other.stack))
+            self.lowest_level_on_line == other.lowest_level_on_line and
+            (self.ignore_stack_for_comparison or
+             other.ignore_stack_for_comparison or self.stack == other.stack))
 
   def __ne__(self, other):
     return not self == other
@@ -125,9 +133,10 @@ class FormatDecisionState(object):
     if current.is_pseudo_paren:
       return False
 
-    if (format_token.Subtype.DICTIONARY_KEY_PART in current.subtypes and
+    if (not must_split and
+        format_token.Subtype.DICTIONARY_KEY_PART in current.subtypes and
         format_token.Subtype.DICTIONARY_KEY not in current.subtypes and
-        not style.Get('ALLOW_MULTILINE_DICTIONARY_KEYS') and not must_split):
+        not style.Get('ALLOW_MULTILINE_DICTIONARY_KEYS')):
       # In some situations, a dictionary may be multiline, but pylint doesn't
       # like it. So don't allow it unless forced to.
       return False
@@ -201,7 +210,8 @@ class FormatDecisionState(object):
       return True
 
     if (format_token.Subtype.DICTIONARY_VALUE in current.subtypes or
-        (previous.is_pseudo_paren and previous.value == '(')):
+        (previous.is_pseudo_paren and previous.value == '(' and
+         not current.is_comment)):
       # Split before the dictionary value if we can't fit every dictionary
       # entry on its own line.
       if not current.OpensScope():
@@ -240,17 +250,8 @@ class FormatDecisionState(object):
             # assigns.
             return False
 
-          func_start = previous.previous_token
-          while func_start and func_start.previous_token:
-            prev = func_start.previous_token
-            if not prev.is_name and prev.value != '.':
-              break
-            func_start = prev
-
-          if func_start:
-            func_name_len = previous.total_length - func_start.total_length
-            func_name_len += len(func_start.value)
-            return func_name_len > style.Get('CONTINUATION_INDENT_WIDTH')
+          column = self.column - self.stack[-1].last_space
+          return column >= style.Get('CONTINUATION_INDENT_WIDTH')
 
         opening = _GetOpeningBracket(current)
         if opening:
@@ -324,6 +325,10 @@ class FormatDecisionState(object):
           # line and it's not a function call.
           if self._FitsOnLine(previous, previous.matching_bracket):
             return False
+        elif not self._FitsOnLine(previous, previous.matching_bracket):
+          if (self.column_limit - self.column) / float(self.column_limit) < 0.3:
+            # Try not to squish all of the arguments off to the right.
+            return current.next_token != previous.matching_bracket
       else:
         # Split after the opening of a container if it doesn't fit on the
         # current line or if it has a comment.
@@ -387,7 +392,7 @@ class FormatDecisionState(object):
     else:
       self._AddTokenOnCurrentLine(dry_run)
 
-    return self._MoveStateToNextToken() + penalty
+    return self.MoveStateToNextToken() + penalty
 
   def _AddTokenOnCurrentLine(self, dry_run):
     """Puts the token on the current line.
@@ -469,12 +474,19 @@ class FormatDecisionState(object):
 
     # Add a penalty for each increasing newline we add, but don't penalize for
     # splitting before an if-expression or list comprehension.
-    if not must_split and current.value not in {'if', 'for'}:
+    if current.value not in {'if', 'for'}:
       last = self.stack[-1]
       last.num_line_splits += 1
       penalty += (style.Get('SPLIT_PENALTY_FOR_ADDED_LINE_SPLIT') *
                   last.num_line_splits)
 
+    if current.OpensScope() and previous.OpensScope():
+      # Prefer to keep opening brackets coalesced (unless it's at the beginning
+      # of a function call).
+      pprev = previous.previous_token
+      if not pprev or not pprev.is_name:
+        penalty += 10
+
     return penalty + 10
 
   def _GetNewlineColumn(self):
@@ -516,7 +528,7 @@ class FormatDecisionState(object):
 
     return top_of_stack.indent
 
-  def _MoveStateToNextToken(self):
+  def MoveStateToNextToken(self):
     """Calculate format decision state information and move onto the next token.
 
     Before moving onto the next token, we first calculate the format decision
@@ -718,7 +730,7 @@ class _ParenState(object):
     self.split_before_closing_bracket = False
     self.num_line_splits = 0
 
-  def __copy__(self):
+  def Clone(self):
     state = _ParenState(self.indent, self.last_space)
     state.closing_scope_indent = self.closing_scope_indent
     state.split_before_closing_bracket = self.split_before_closing_bracket
