@@ -56,6 +56,7 @@ def Reformat(uwlines, verify=False):
 
     indent_amt = indent_width * uwline.depth
     state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+    state.MoveStateToNextToken()
 
     if not uwline.disable:
       if uwline.first.is_comment:
@@ -84,6 +85,7 @@ def Reformat(uwlines, verify=False):
         # Failsafe mode. If there isn't a solution to the line, then just emit
         # it as is.
         state = format_decision_state.FormatDecisionState(uwline, indent_amt)
+        state.MoveStateToNextToken()
         _RetainHorizontalSpacing(uwline)
         _RetainVerticalSpacing(uwline, prev_uwline)
         _EmitLineUnformatted(state)
@@ -219,8 +221,7 @@ def _CanPlaceOnSingleLine(uwline):
   indent_amt = style.Get('INDENT_WIDTH') * uwline.depth
   last = uwline.last
   last_index = -1
-  if last.is_comment and re.search(r'^#+\s+pylint:',
-                                   last.value.strip(), re.IGNORECASE):
+  if last.is_pylint_comment:
     last = last.previous_token
     last_index = -2
   if last is None:
@@ -271,8 +272,8 @@ class _StateNode(object):
     self.previous = previous
 
   def __repr__(self):  # pragma: no cover
-    return 'StateNode(state=[\n{0}\n], newline={1})'.format(self.state,
-                                                            self.newline)
+    return 'StateNode(state=[\n{0}\n], newline={1})'.format(
+        self.state, self.newline)
 
 
 # A tuple of (penalty, count) that is used to prioritize the BFS. In case of
