@@ -145,8 +145,8 @@ class UnwrappedLine(object):
   def __repr__(self):  # pragma: no cover
     tokens_repr = ','.join(
         ['{0}({1!r})'.format(tok.name, tok.value) for tok in self._tokens])
-    return 'UnwrappedLine(depth={0}, tokens=[{1}])'.format(self.depth,
-                                                           tokens_repr)
+    return 'UnwrappedLine(depth={0}, tokens=[{1}])'.format(
+        self.depth, tokens_repr)
 
   ############################################################################
   # Properties                                                               #
@@ -273,7 +273,7 @@ def _SpaceRequiredBetween(left, right):
       format_token.Subtype.KWARGS_STAR_STAR in left.subtypes):
     # Don't add a space after a vararg's star or a keyword's star-star.
     return False
-  if (lval == '@' and format_token.Subtype.DECORATOR in left.subtypes):
+  if lval == '@' and format_token.Subtype.DECORATOR in left.subtypes:
     # Decorators shouldn't be separated from the 'at' sign.
     return False
   if lval == '.' or rval == '.':
