@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2016 Google Inc. All Rights Reserved.
+# Copyright 2015-2017 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,8 +25,11 @@ import textwrap
 import unittest
 
 from yapf.yapflib import py3compat
+from yapf.yapflib import style
 from yapf.yapflib import yapf_api
 
+from yapftests import utils
+
 ROOT_DIR = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
 
 # Verification is turned off by default, but want to enable it for testing.
@@ -41,16 +44,16 @@ class FormatCodeTest(unittest.TestCase):
     self.assertEqual(expected_formatted_code, formatted_code)
 
   def testSimple(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         print('foo')
         """)
     self._Check(unformatted_code, unformatted_code)
 
   def testNoEndingNewline(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         if True:
           pass""")
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         if True:
           pass
         """)
@@ -75,19 +78,11 @@ class FormatFileTest(unittest.TestCase):
       # TODO(sbc): maybe using difflib here to produce easy to read deltas?
       self.fail(msg)
 
-  def _MakeTempFileWithContents(self, filename, contents):
-    path = os.path.join(self.test_tmpdir, filename)
-    with open(path, 'w') as f:
-      f.write(contents)
-    return path
-
   def testFormatFile(self):
     unformatted_code = textwrap.dedent(u"""\
         if True:
          pass
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-
     expected_formatted_code_pep8 = textwrap.dedent(u"""\
         if True:
             pass
@@ -96,12 +91,13 @@ class FormatFileTest(unittest.TestCase):
         if True:
           pass
         """)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code_pep8, formatted_code)
 
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(expected_formatted_code_pep8, formatted_code)
-
-    formatted_code = yapf_api.FormatFile(file1, style_config='chromium')[0]
-    self.assertCodeEqual(expected_formatted_code_chromium, formatted_code)
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='chromium')
+      self.assertCodeEqual(expected_formatted_code_chromium, formatted_code)
 
   def testDisableLinesPattern(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -112,8 +108,6 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-
     expected_formatted_code = textwrap.dedent(u"""\
         if a: b
 
@@ -122,8 +116,9 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(expected_formatted_code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code, formatted_code)
 
   def testDisableAndReenableLinesPattern(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -135,8 +130,6 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-
     expected_formatted_code = textwrap.dedent(u"""\
         if a: b
 
@@ -146,8 +139,9 @@ class FormatFileTest(unittest.TestCase):
 
         if h: i
         """)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(expected_formatted_code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code, formatted_code)
 
   def testDisablePartOfMultilineComment(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -161,7 +155,6 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
 
     expected_formatted_code = textwrap.dedent(u"""\
         if a: b
@@ -174,10 +167,11 @@ class FormatFileTest(unittest.TestCase):
 
         if h: i
         """)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(expected_formatted_code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(expected_formatted_code, formatted_code)
 
-    code = textwrap.dedent("""\
+    code = textwrap.dedent(u"""\
       def foo_function():
           # some comment
           # yapf: disable
@@ -189,9 +183,9 @@ class FormatFileTest(unittest.TestCase):
 
           # yapf: enable
       """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', code)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(code, formatted_code)
 
   def testFormatFileLinesSelection(self):
     unformatted_code = textwrap.dedent(u"""\
@@ -201,8 +195,6 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-
     expected_formatted_code_lines1and2 = textwrap.dedent(u"""\
         if a: b
 
@@ -210,10 +202,6 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    formatted_code = yapf_api.FormatFile(
-        file1, style_config='pep8', lines=[(1, 2)])[0]
-    self.assertCodeEqual(expected_formatted_code_lines1and2, formatted_code)
-
     expected_formatted_code_lines3 = textwrap.dedent(u"""\
         if a:    b
 
@@ -221,30 +209,41 @@ class FormatFileTest(unittest.TestCase):
 
         if h:    i
         """)
-    formatted_code = yapf_api.FormatFile(
-        file1, style_config='pep8', lines=[(3, 3)])[0]
-    self.assertCodeEqual(expected_formatted_code_lines3, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='pep8', lines=[(1, 2)])
+      self.assertCodeEqual(expected_formatted_code_lines1and2, formatted_code)
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='pep8', lines=[(3, 3)])
+      self.assertCodeEqual(expected_formatted_code_lines3, formatted_code)
 
   def testFormatFileDiff(self):
     unformatted_code = textwrap.dedent(u"""\
         if True:
          pass
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-    diff = yapf_api.FormatFile(file1, print_diff=True)[0]
-    self.assertTrue(u'+    pass' in diff)
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      diff, _, _ = yapf_api.FormatFile(filepath, print_diff=True)
+      self.assertTrue(u'+  pass' in diff)
 
   def testFormatFileInPlace(self):
-    unformatted_code = 'True==False\n'
-    formatted_code = 'True == False\n'
-    file1 = self._MakeTempFileWithContents('testfile1.py', unformatted_code)
-    result, _, _ = yapf_api.FormatFile(file1, in_place=True)
-    self.assertEqual(result, None)
-    with open(file1) as f:
-      self.assertCodeEqual(formatted_code, f.read())
+    unformatted_code = u'True==False\n'
+    formatted_code = u'True == False\n'
+    with utils.TempFileContents(self.test_tmpdir, unformatted_code) as filepath:
+      result, _, _ = yapf_api.FormatFile(filepath, in_place=True)
+      self.assertEqual(result, None)
+      with open(filepath) as fd:
+        if sys.version_info[0] <= 2:
+          self.assertCodeEqual(formatted_code, fd.read().decode('ascii'))
+        else:
+          self.assertCodeEqual(formatted_code, fd.read())
 
-    self.assertRaises(
-        ValueError, yapf_api.FormatFile, file1, in_place=True, print_diff=True)
+      self.assertRaises(
+          ValueError,
+          yapf_api.FormatFile,
+          filepath,
+          in_place=True,
+          print_diff=True)
 
   def testNoFile(self):
     stream = py3compat.StringIO()
@@ -257,39 +256,39 @@ class FormatFileTest(unittest.TestCase):
                      "[Errno 2] No such file or directory: 'not_a_file.py'\n")
 
   def testCommentsUnformatted(self):
-    code = textwrap.dedent("""\
+    code = textwrap.dedent(u"""\
         foo = [# A list of things
                # bork
             'one',
             # quark
             'two'] # yapf: disable
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', code)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(code, formatted_code)
 
   def testDisabledHorizontalFormattingOnNewLine(self):
-    code = textwrap.dedent("""\
+    code = textwrap.dedent(u"""\
         # yapf: disable
         a = [
         1]
         # yapf: enable
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', code)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(code, formatted_code)
 
   def testDisabledSemiColonSeparatedStatements(self):
-    code = textwrap.dedent("""\
+    code = textwrap.dedent(u"""\
         # yapf: disable
         if True: a ; b
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', code)
-    formatted_code = yapf_api.FormatFile(file1, style_config='pep8')[0]
-    self.assertCodeEqual(code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(filepath, style_config='pep8')
+      self.assertCodeEqual(code, formatted_code)
 
   def testDisabledMultilineStringInDictionary(self):
-    code = textwrap.dedent("""\
+    code = textwrap.dedent(u"""\
         # yapf: disable
 
         A = [
@@ -303,9 +302,37 @@ class FormatFileTest(unittest.TestCase):
             },
         ]
         """)
-    file1 = self._MakeTempFileWithContents('testfile1.py', code)
-    formatted_code = yapf_api.FormatFile(file1, style_config='chromium')[0]
-    self.assertCodeEqual(code, formatted_code)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='chromium')
+      self.assertCodeEqual(code, formatted_code)
+
+  def testDisabledWithPrecedingText(self):
+    code = textwrap.dedent(u"""\
+        # TODO(fix formatting): yapf: disable
+
+        A = [
+            {
+                "aaaaaaaaaaaaaaaaaaa": '''
+        bbbbbbbbbbb: "ccccccccccc"
+        dddddddddddddd: 1
+        eeeeeeee: 0
+        ffffffffff: "ggggggg"
+        ''',
+            },
+        ]
+        """)
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='chromium')
+      self.assertCodeEqual(code, formatted_code)
+
+  def testCRLFLineEnding(self):
+    code = u'class _():\r\n  pass\r\n'
+    with utils.TempFileContents(self.test_tmpdir, code) as filepath:
+      formatted_code, _, _ = yapf_api.FormatFile(
+          filepath, style_config='chromium')
+      self.assertCodeEqual(code, formatted_code)
 
 
 class CommandLineTest(unittest.TestCase):
@@ -345,98 +372,77 @@ class CommandLineTest(unittest.TestCase):
         def foo():
             print('⇒')
         """)
-
-    with tempfile.NamedTemporaryFile(
-        suffix='.py', dir=self.test_tmpdir) as outfile:
-      with tempfile.NamedTemporaryFile(
-          suffix='.py', dir=self.test_tmpdir) as testfile:
-        testfile.write(unformatted_code.encode('UTF-8'))
-        subprocess.check_call(
-            YAPF_BINARY + ['--diff', testfile.name], stdout=outfile)
+    with utils.NamedTempFile(
+        dirname=self.test_tmpdir, suffix='.py') as (out, _):
+      with utils.TempFileContents(
+          self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
+        subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
 
   def testInPlaceReformatting(self):
     unformatted_code = textwrap.dedent(u"""\
         def foo():
           x = 37
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():
             x = 37
         """)
-
-    with tempfile.NamedTemporaryFile(
-        suffix='.py', dir=self.test_tmpdir) as testfile:
-      testfile.write(unformatted_code.encode('UTF-8'))
-      testfile.seek(0)
-
-      p = subprocess.Popen(YAPF_BINARY + ['--in-place', testfile.name])
+    with utils.TempFileContents(
+        self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
+      p = subprocess.Popen(YAPF_BINARY + ['--in-place', filepath])
       p.wait()
-
-      with io.open(testfile.name, mode='r', newline='') as fd:
+      with io.open(filepath, mode='r', newline='') as fd:
         reformatted_code = fd.read()
-
     self.assertEqual(reformatted_code, expected_formatted_code)
 
   def testInPlaceReformattingBlank(self):
     unformatted_code = u'\n\n'
     expected_formatted_code = u'\n'
-
-    with tempfile.NamedTemporaryFile(
-        suffix='.py', dir=self.test_tmpdir) as testfile:
-      testfile.write(unformatted_code.encode('UTF-8'))
-      testfile.seek(0)
-
-      p = subprocess.Popen(YAPF_BINARY + ['--in-place', testfile.name])
+    with utils.TempFileContents(
+        self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
+      p = subprocess.Popen(YAPF_BINARY + ['--in-place', filepath])
       p.wait()
-
-      with io.open(testfile.name, mode='r', newline='') as fd:
+      with io.open(filepath, mode='r', encoding='utf-8', newline='') as fd:
         reformatted_code = fd.read()
-
     self.assertEqual(reformatted_code, expected_formatted_code)
 
   def testInPlaceReformattingEmpty(self):
     unformatted_code = u''
     expected_formatted_code = u''
-
-    with tempfile.NamedTemporaryFile(
-        suffix='.py', dir=self.test_tmpdir) as testfile:
-      testfile.write(unformatted_code.encode('UTF-8'))
-      testfile.seek(0)
-
-      p = subprocess.Popen(YAPF_BINARY + ['--in-place', testfile.name])
+    with utils.TempFileContents(
+        self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
+      p = subprocess.Popen(YAPF_BINARY + ['--in-place', filepath])
       p.wait()
-
-      with io.open(testfile.name, mode='r', newline='') as fd:
+      with io.open(filepath, mode='r', encoding='utf-8', newline='') as fd:
         reformatted_code = fd.read()
-
     self.assertEqual(reformatted_code, expected_formatted_code)
 
   def testReadFromStdin(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo():
           x = 37
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():
             x = 37
         """)
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testReadFromStdinWithEscapedStrings(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         s =   "foo\\nbar"
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         s = "foo\\nbar"
         """)
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testSetChromiumStyle(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo(): # trail
             x = 37
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():  # trail
           x = 37
         """)
@@ -446,33 +452,30 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--style=chromium'])
 
   def testSetCustomStyleBasedOnChromium(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo(): # trail
             x = 37
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():    # trail
           x = 37
         """)
-
-    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
-      f.write(
-          textwrap.dedent('''\
-          [style]
-          based_on_style = chromium
-          SPACES_BEFORE_COMMENT = 4
-          '''))
-      f.flush()
+    style_file = textwrap.dedent(u'''\
+        [style]
+        based_on_style = chromium
+        SPACES_BEFORE_COMMENT = 4
+        ''')
+    with utils.TempFileContents(self.test_tmpdir, style_file) as stylepath:
       self.assertYapfReformats(
           unformatted_code,
           expected_formatted_code,
-          extra_options=['--style={0}'.format(f.name)])
+          extra_options=['--style={0}'.format(stylepath)])
 
   def testReadSingleLineCodeFromStdin(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         if True: pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         if True: pass
         """)
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
@@ -485,16 +488,14 @@ class CommandLineTest(unittest.TestCase):
             x = 37
         """)
 
-    with tempfile.NamedTemporaryFile(
-        suffix='.py', dir=self.test_tmpdir) as outfile:
-      with tempfile.NamedTemporaryFile(
-          suffix='.py', dir=self.test_tmpdir) as testfile:
-        testfile.write(unformatted_code.encode('utf-8'))
-        subprocess.check_call(
-            YAPF_BINARY + ['--diff', testfile.name], stdout=outfile)
+    with utils.NamedTempFile(
+        suffix='.py', dirname=self.test_tmpdir) as (out, _):
+      with utils.TempFileContents(
+          self.test_tmpdir, unformatted_code, suffix='.py') as filepath:
+        subprocess.check_call(YAPF_BINARY + ['--diff', filepath], stdout=out)
 
   def testReformattingSpecificLines(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -504,7 +505,7 @@ class CommandLineTest(unittest.TestCase):
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -523,8 +524,28 @@ class CommandLineTest(unittest.TestCase):
         expected_formatted_code,
         extra_options=['--lines', '1-2'])
 
+  def testOmitFormattingLinesBeforeDisabledFunctionComment(self):
+    unformatted_code = textwrap.dedent("""\
+        import sys
+
+        # Comment
+        def some_func(x):
+            x = ["badly" , "formatted","line" ]
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        import sys
+
+        # Comment
+        def some_func(x):
+            x = ["badly", "formatted", "line"]
+        """)
+    self.assertYapfReformats(
+        unformatted_code,
+        expected_formatted_code,
+        extra_options=['--lines', '5-5'])
+
   def testReformattingSkippingLines(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -535,7 +556,7 @@ class CommandLineTest(unittest.TestCase):
                 pass
         # yapf: enable
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -551,7 +572,7 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testReformattingSkippingToEndOfFile(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -568,7 +589,7 @@ class CommandLineTest(unittest.TestCase):
                        'bbbbbbb'):
                     pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -590,7 +611,7 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testReformattingSkippingSingleLine(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -599,7 +620,7 @@ class CommandLineTest(unittest.TestCase):
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):  # yapf: disable
                 pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -613,13 +634,13 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testDisableWholeDataStructure(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         A = set([
             'hello',
             'world',
         ])  # yapf: disable
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         A = set([
             'hello',
             'world',
@@ -628,13 +649,13 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testDisableButAdjustIndentations(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         class SplitPenaltyTest(unittest.TestCase):
           def testUnbreakable(self):
             self._CheckPenalties(tree, [
             ])  # yapf: disable
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         class SplitPenaltyTest(unittest.TestCase):
             def testUnbreakable(self):
                 self._CheckPenalties(tree, [
@@ -643,7 +664,7 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testRetainingHorizontalWhitespace(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -652,7 +673,7 @@ class CommandLineTest(unittest.TestCase):
             if (xxxxxxxxxxxx.yyyyyyyy        (zzzzzzzzzzzzz  [0]) ==     'aaaaaaaaaaa' and    xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):  # yapf: disable
                 pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -666,7 +687,7 @@ class CommandLineTest(unittest.TestCase):
     self.assertYapfReformats(unformatted_code, expected_formatted_code)
 
   def testRetainingVerticalWhitespace(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
                 pass
@@ -678,7 +699,7 @@ class CommandLineTest(unittest.TestCase):
 
                 pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def h():
             if (xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0]) == 'aaaaaaaaaaa' and
                     xxxxxxxxxxxx.yyyyyyyy(zzzzzzzzzzzzz[0].mmmmmmmm[0]) == 'bbbbbbb'):
@@ -696,7 +717,7 @@ class CommandLineTest(unittest.TestCase):
         expected_formatted_code,
         extra_options=['--lines', '1-2'])
 
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
 
 
         if a:     b
@@ -713,7 +734,7 @@ class CommandLineTest(unittest.TestCase):
 
         #   trailing whitespace
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         if a: b
 
 
@@ -733,7 +754,7 @@ class CommandLineTest(unittest.TestCase):
         expected_formatted_code,
         extra_options=['--lines', '3-3', '--lines', '13-13'])
 
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         '''
         docstring
 
@@ -790,7 +811,7 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '1-1'])
 
   def testDisableWhenSpecifyingLines(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         # yapf: disable
         A = set([
             'hello',
@@ -802,7 +823,7 @@ class CommandLineTest(unittest.TestCase):
             'world',
         ])  # yapf: disable
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         # yapf: disable
         A = set([
             'hello',
@@ -820,7 +841,7 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '1-10'])
 
   def testDisableFormattingInDataLiteral(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def horrible():
           oh_god()
           why_would_you()
@@ -839,7 +860,7 @@ class CommandLineTest(unittest.TestCase):
                 'that'
             ]
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def horrible():
             oh_god()
             why_would_you()
@@ -860,7 +881,7 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '14-15'])
 
   def testRetainVerticalFormattingBetweenDisabledAndEnabledLines(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         class A(object):
             def aaaaaaaaaaaaa(self):
                 c = bbbbbbbbb.ccccccccc('challenge', 0, 1, 10)
@@ -871,7 +892,7 @@ class CommandLineTest(unittest.TestCase):
              gggggggggggg.hhhhhhhhh(c, c.ffffffffffff))
                 iiiii = jjjjjjjjjjjjjj.iiiii
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         class A(object):
             def aaaaaaaaaaaaa(self):
                 c = bbbbbbbbb.ccccccccc('challenge', 0, 1, 10)
@@ -886,7 +907,7 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '4-7'])
 
   def testFormatLinesSpecifiedInMiddleOfExpression(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         class A(object):
             def aaaaaaaaaaaaa(self):
                 c = bbbbbbbbb.ccccccccc('challenge', 0, 1, 10)
@@ -897,7 +918,7 @@ class CommandLineTest(unittest.TestCase):
              gggggggggggg.hhhhhhhhh(c, c.ffffffffffff))
                 iiiii = jjjjjjjjjjjjjj.iiiii
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         class A(object):
             def aaaaaaaaaaaaa(self):
                 c = bbbbbbbbb.ccccccccc('challenge', 0, 1, 10)
@@ -912,7 +933,7 @@ class CommandLineTest(unittest.TestCase):
         extra_options=['--lines', '5-6'])
 
   def testCommentFollowingMultilineString(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo():
             '''First line.
             Second line.
@@ -920,7 +941,7 @@ class CommandLineTest(unittest.TestCase):
             x = '''hello world'''  # second comment
             return 42  # another comment
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():
             '''First line.
             Second line.
@@ -935,12 +956,12 @@ class CommandLineTest(unittest.TestCase):
 
   def testDedentClosingBracket(self):
     # no line-break on the first argument, not dedenting closing brackets
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
       def overly_long_function_name(first_argument_on_the_same_line,
       second_argument_makes_the_line_too_long):
         pass
     """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
       def overly_long_function_name(first_argument_on_the_same_line,
                                     second_argument_makes_the_line_too_long):
           pass
@@ -958,19 +979,19 @@ class CommandLineTest(unittest.TestCase):
     #                          extra_options=['--style=facebook'])
 
     # line-break before the first argument, dedenting closing brackets if set
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
       def overly_long_function_name(
         first_argument_on_the_same_line,
         second_argument_makes_the_line_too_long):
         pass
     """)
-    expected_formatted_pep8_code = textwrap.dedent(u"""\
-      def overly_long_function_name(
-              first_argument_on_the_same_line, \
-second_argument_makes_the_line_too_long):
-          pass
-    """)
-    expected_formatted_fb_code = textwrap.dedent(u"""\
+    # expected_formatted_pep8_code = textwrap.dedent("""\
+    #   def overly_long_function_name(
+    #           first_argument_on_the_same_line,
+    #           second_argument_makes_the_line_too_long):
+    #       pass
+    # """)
+    expected_formatted_fb_code = textwrap.dedent("""\
       def overly_long_function_name(
           first_argument_on_the_same_line, second_argument_makes_the_line_too_long
       ):
@@ -988,20 +1009,20 @@ second_argument_makes_the_line_too_long):
     #                          extra_options=['--style=pep8'])
 
   def testCoalesceBrackets(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
        some_long_function_name_foo({
            'first_argument_of_the_thing': id,
            'second_argument_of_the_thing': "some thing"}
            )""")
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
        some_long_function_name_foo({
            'first_argument_of_the_thing': id,
            'second_argument_of_the_thing': "some thing"
        })
        """)
-    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
+    with utils.NamedTempFile(dirname=self.test_tmpdir, mode='w') as (f, name):
       f.write(
-          textwrap.dedent('''\
+          textwrap.dedent(u'''\
           [style]
           based_on_style = facebook
           column_limit=82
@@ -1011,15 +1032,15 @@ second_argument_makes_the_line_too_long):
       self.assertYapfReformats(
           unformatted_code,
           expected_formatted_code,
-          extra_options=['--style={0}'.format(f.name)])
+          extra_options=['--style={0}'.format(name)])
 
   def testPseudoParenSpaces(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo():
           def bar():
             return {msg_id: author for author, msg_id in reader}
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo():
           def bar():
             return {msg_id: author for author, msg_id in reader}
@@ -1030,7 +1051,7 @@ second_argument_makes_the_line_too_long):
         extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testMultilineCommentFormattingDisabled(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         # This is a comment
         FOO = {
             aaaaaaaa.ZZZ: [
@@ -1044,7 +1065,7 @@ second_argument_makes_the_line_too_long):
             '#': lambda x: x  # do nothing
         }
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         # This is a comment
         FOO = {
             aaaaaaaa.ZZZ: [
@@ -1064,14 +1085,14 @@ second_argument_makes_the_line_too_long):
         extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testTrailingCommentsWithDisabledFormatting(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         import os
 
         SCOPES = [
             'hello world'  # This is a comment.
         ]
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         import os
 
         SCOPES = [
@@ -1084,29 +1105,51 @@ second_argument_makes_the_line_too_long):
         extra_options=['--lines', '1-1', '--style', 'chromium'])
 
   def testUseTabs(self):
-    unformatted_code = textwrap.dedent(u"""\
+    unformatted_code = textwrap.dedent("""\
         def foo_function():
          if True:
           pass
         """)
-    expected_formatted_code = textwrap.dedent(u"""\
+    expected_formatted_code = textwrap.dedent("""\
         def foo_function():
         	if True:
         		pass
         """)
-    with tempfile.NamedTemporaryFile(dir=self.test_tmpdir, mode='w') as f:
-      f.write(
-          textwrap.dedent('''\
-          [style]
-          based_on_style = chromium
-          USE_TABS = true
-          INDENT_WIDTH=1
-          '''))
-      f.flush()
+    style_contents = textwrap.dedent(u'''\
+        [style]
+        based_on_style = chromium
+        USE_TABS = true
+        INDENT_WIDTH=1
+        ''')
+    with utils.TempFileContents(self.test_tmpdir, style_contents) as stylepath:
       self.assertYapfReformats(
           unformatted_code,
           expected_formatted_code,
-          extra_options=['--style={0}'.format(f.name)])
+          extra_options=['--style={0}'.format(stylepath)])
+
+  def testStyleOutputRoundTrip(self):
+    unformatted_code = textwrap.dedent("""\
+        def foo_function():
+          pass
+        """)
+    expected_formatted_code = textwrap.dedent("""\
+        def foo_function():
+            pass
+        """)
+
+    with utils.NamedTempFile(dirname=self.test_tmpdir) as (stylefile,
+                                                           stylepath):
+      p = subprocess.Popen(
+          YAPF_BINARY + ['--style-help'],
+          stdout=stylefile,
+          stdin=subprocess.PIPE,
+          stderr=subprocess.PIPE)
+      _, stderrdata = p.communicate()
+      self.assertEqual(stderrdata, b'')
+      self.assertYapfReformats(
+          unformatted_code,
+          expected_formatted_code,
+          extra_options=['--style={0}'.format(stylepath)])
 
 
 class BadInputTest(unittest.TestCase):
@@ -1117,5 +1160,31 @@ class BadInputTest(unittest.TestCase):
     self.assertRaises(SyntaxError, yapf_api.FormatCode, code)
 
 
+class DiffIndentTest(unittest.TestCase):
+
+  @staticmethod
+  def _OwnStyle():
+    my_style = style.CreatePEP8Style()
+    my_style['INDENT_WIDTH'] = 3
+    my_style['CONTINUATION_INDENT_WIDTH'] = 3
+    return my_style
+
+  def _Check(self, unformatted_code, expected_formatted_code):
+    formatted_code, _ = yapf_api.FormatCode(
+        unformatted_code, style_config=style.SetGlobalStyle(self._OwnStyle()))
+    self.assertEqual(expected_formatted_code, formatted_code)
+
+  def testSimple(self):
+    unformatted_code = textwrap.dedent("""\
+        for i in range(5):
+         print('bar')
+         """)
+    expected_formatted_code = textwrap.dedent("""\
+        for i in range(5):
+           print('bar')
+           """)
+    self._Check(unformatted_code, expected_formatted_code)
+
+
 if __name__ == '__main__':
   unittest.main()
